"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const parser_1 = require("./parser");
const default_flub_options_1 = require("./default-flub-options");
const fs = require("fs");
const Mustache = require("mustache");
const path = require("path");
class ErrorHandler {
    constructor(error, options = new default_flub_options_1.DefaultFlubOptions()) {
        this.error = error;
        this.options = options;
        this.errorParser = new parser_1.ErrorParser(error, options);
    }
    /**
     * Returns error stack as JSON.
     *
     * @return {Promise}
     */
    toJSON() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.errorParser
                    .parse()
                    .then(stack => {
                    resolve({
                        error: this.errorParser.serialize(stack),
                    });
                })
                    .catch(reject);
            });
        });
    }
    /**
     * Returns HTML representation of the error stack
     * by parsing the stack into frames and getting
     * important info out of it.
     *
     * @return {Promise}
     */
    toHTML() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.errorParser
                    .parse()
                    .then(stack => {
                    const data = this.errorParser.serialize(stack, (frame, index) => {
                        const serializedFrame = parser_1.FrameParser.serializeCodeFrame(frame);
                        serializedFrame.classes = this.getDisplayClasses(frame, index);
                        return serializedFrame;
                    });
                    const viewTemplate = fs.readFileSync(path.join(__dirname, `./themes/error.${this.options.theme || 'default'}.mustache`), 'utf-8');
                    resolve(this.complieView(viewTemplate, data));
                })
                    .catch(reject);
            });
        });
    }
    /**
     * Returns classes to be used inside HTML when
     * displaying the frames list.
     *
     * @param  {Object}
     * @param  {Number}
     *
     * @return {String}
     */
    getDisplayClasses(frame, index) {
        const classes = [];
        if (index === 0) {
            classes.push('active');
        }
        if (!parser_1.FrameParser.isApp(frame)) {
            classes.push('native-frame');
        }
        return classes.join(' ');
    }
    /**
     * Compiles the view using HTML
     *
     * @param  {String}
     * @param  {Object}
     *
     * @return {String}
     */
    complieView(view, data) {
        return Mustache.render(view, data);
    }
}
exports.ErrorHandler = ErrorHandler;
