"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stackTrace = require("stack-trace");
const quotes_1 = require("./../quotes");
const frame_parser_1 = require("./frame-parser");
class ErrorParser {
    constructor(error, options) {
        this.viewQuote = true;
        this.error = error;
        this.viewQuote = options.quote;
    }
    /**
     * Serializes stack to Mustache friendly object to
     * be used within the view. Optionally can pass
     * a callback to customize the frames output.
     *
     * @param  {Object} stack
     * @param  {Function} [callback]
     *
     * @return {Object}
     */
    serialize(stack, callback) {
        callback = callback || frame_parser_1.FrameParser.serializeCodeFrame.bind(this);
        let frames = [];
        if (stack instanceof Array) {
            frames = stack.filter(frame => frame.getFileName()).map(callback);
        }
        return {
            frames,
            message: this.error.message,
            name: this.error.name,
            quote: this.viewQuote ? this.randomQuote() : undefined,
        };
    }
    /**
     * Parses the error stack and returns serialized
     * frames out of it.
     *
     * @return {Promise<Object>}
     */
    parse() {
        return new Promise((resolve, reject) => {
            const stack = stackTrace.parse(this.error);
            Promise.all(stack.map(frame => {
                if (frame_parser_1.FrameParser.isNode(frame)) {
                    return Promise.resolve(frame);
                }
                return frame_parser_1.FrameParser.readCodeFrame(frame).then(context => {
                    frame.context = context;
                    return frame;
                });
            }))
                .then(resolve)
                .catch(reject);
        });
    }
    /**
     * @return string
     */
    randomQuote() {
        return quotes_1.default[Math.floor(Math.random() * quotes_1.default.length)];
    }
}
exports.ErrorParser = ErrorParser;
